library finding_snippets;

import 'dart:html';
import 'package:unittest/unittest.dart';

run() {
  group("[finding]", (){
    var el = new DivElement();
    setUp((){
      document.body.nodes.add(el);
      el.innerHtml = """
        <h1>First Big Title</h1>
        <h2>First Small Title</h2>
        <h2>Second Small Title</h2>
        <h2>Third Small Title</h2>

        <h1>Second Big Title</h1>
        <ul id=people-list>
          <li>Bob</li>
          <li class=active>Sally</li>
          <li class=active>Lucy</li>
        </ul>
      """;
    });
    tearDown((){
      document.body.nodes.removeLast();
      el.innerHtml = '';
    });

    test('query single tag', (){
      var el =
        // START:all
        // START:one_by_tag
        document.query('h1');
        // => First <h1> in the document
        // END:one_by_tag
        // END:all

      expect(el.innerHtml, equals('First Big Title'));
    });

    test('query single ID', (){
      var el =
        // START:all
        // START:one_by_id
        document.query('#people-list');
        // => Element with id of 'people-list'
        // END:one_by_id
        // END:all

      expect(el.tagName, equals('UL'));
      expect(el.children.length, equals(3));
    });

    test('query single class', (){
      var el =
        // START:all
        // START:one_by_class
        document.query('.active');
        // => First element with 'active' class
        // END:one_by_class
        // END:all

      expect(el.innerHtml, equals('Sally'));
    });

    test('query single class', (){
      var els =
        // START:all
        // START:all_by_tag
        document.queryAll('h2');
        // => All <h2> elements
        // END:all_by_tag
        // END:all

      expect(els.length, equals(3));
      expect(els[2].innerHtml, equals('Third Small Title'));
    });

    test('scope queries to other queuries', (){
      // START:scope_queries
      var list = document.query('ul#people-list');
      var last_person = list.query(':last-child');
      last_person.innerHtml;
      // => 'Lucy'
      // END:scope_queries

      expect(last_person.innerHtml, equals('Lucy'));
    });

    test('scope chains', (){
      var last_person =
      // START:scope_chain
      document.
        query('ul#people-list').
        query(':last-child').
        innerHtml;
      // => 'Lucy'
      // END:scope_chain

      expect(last_person, equals('Lucy'));
    });

    test('iterating', (){
      // START:iterating
      document.
        query('ul#people-list').
        queryAll('li').
        forEach((li) {
          li.classes.add('highlight');
        });
      // END:iterating

      expect(document.queryAll('li').last.classes, contains('highlight'));
    });
  });
}
