library updating_snippets;

import 'dart:html';
import 'package:unittest/unittest.dart';

run() {
  group("[updating]", (){
    var el = new DivElement();

    setUp((){
      document.body.nodes.add(el);
    });
    tearDown((){
      document.body.nodes.removeLast();
      el.innerHtml = '';
    });

    test('adding and removing a class', (){
      el.append(new Element.html('''
        <blockquote class="subdued"></blockquote>
      '''));

      // START:add_remove
      document.
        query('blockquote').
        classes.
        remove('subdued');

      document.
        query('blockquote').
        classes.
        add('highlighted');
      // END:add_remove

      expect(el.query('blockquote').classes, contains('highlighted'));
      expect(el.query('blockquote').classes, isNot(contains('subdued')));
    });

    test('updating content', (){
      el.append(new Element.html('''
        <blockquote></blockquote>
      '''));

      // START:innerHtml
      document.
        query('blockquote').
        innerHtml = 'Four score and <u>seven</u> years ago...';
      // END:innerHtml

      expect(el.query('blockquote').innerHtml, equals('Four score and <u>seven</u> years ago...'));
    });

    test('method cascade', (){
      el.append(new Element.html('''
        <blockquote class="subdued"></blockquote>
      '''));

      // START:cascade
      document.
        query('blockquote').
        classes
          ..remove('subdued')
          ..add('highlighted');
      // END:cascade

      expect(el.query('blockquote').classes, contains('highlighted'));
      expect(el.query('blockquote').classes, isNot(contains('subdued')));
    });


  });
}
