library fib_snippet;

import 'package:unittest/unittest.dart';

// START:fib_function_with_comment
// Dart
// START:fib_function
fib(i) {
  if (i < 2) return i;
  return fib(i-2) + fib(i-1);
}
// END:fib_function
// END:fib_function_with_comment

var _prints = [];
print(val) => _prints.add(val);

// START:fib_printer
fib_printer(i) {
  print("Fib($i): ${fib(i)}");
}
// END:fib_printer

var anon =
// START:anonymous_fib_function
(i) {
  if (i < 2) return i;
  return fib(i-2) + fib(i-1);
}
// END:anonymous_fib_function
;

run() {
  group("[fib]", (){
    setUp((){
      /* ------------------------- */
      /* ------------------------- */
    });

    test('fib(1); // => 1', (){
      expect(fib(1), equals(1));
    });

    test('fib(5); // => 5', (){
      expect(fib(5), equals(5));
    });

    test('fib(10); // => 55', (){
      expect(fib(10), equals(55));
    });

    test('anonymous functions work too!', (){
      expect(anon(1), equals(1));
      expect(anon(5), equals(5));
      expect(anon(10), equals(55));
    });

    test('simple forEach example', (){
      _prints = [];
      // START:simple_forEach
      var list = [1, 5, 8, 10];
      list.forEach((i) {fib_printer(i);});
      // END:simple_forEach

      expect(_prints, equals([
        'Fib(1): 1',
        'Fib(5): 5',
        'Fib(8): 21',
        'Fib(10): 55'
      ]));
    });

    test('hash rockets make everything nicer', (){
      _prints = [];
      // START:hash_rocket
      var list = [1, 5, 8, 10];
      list.forEach((i) => fib_printer(i));
      // END:hash_rocket

      expect(_prints, equals([
        'Fib(1): 1',
        'Fib(5): 5',
        'Fib(8): 21',
        'Fib(10): 55'
      ]));
    });

    test('omit dupe parameter in hash rocket', (){
      _prints = [];
      // START:omit_dupe_params
      var list = [1, 5, 8, 10];
      list.forEach(fib_printer);
      // END:omit_dupe_params

      expect(_prints, equals([
        'Fib(1): 1',
        'Fib(5): 5',
        'Fib(8): 21',
        'Fib(10): 55'
      ]));
    });
  });
}
