library optional_snippet;

import 'package:unittest/unittest.dart';

var _prints = [];
print(val) => _prints.add(val);

// START:good_day
good_day(name, {emphatic}) {
  print("Good day, ${name}.");
  if (emphatic) {
    print("I said good day!");
  }
}
// END:good_day

// START:profile
profile(name, {hero:"Weird Al", favorite_color}) {
  print("Name: ${name}");
  print("  personal hero: ${hero}");
  if (favorite_color != null) {
    print("  favorite color: ${favorite_color}");
  }
}
// END:profile

// START:movie
movie(title, [starring="Leslie Nielson", co_starring]) {
  print("Great movie: ${title}");
  print("  Starring: ${starring}");
  if (co_starring != null) {
    print("  Co-starring: ${co_starring}");
  }
}
// END:movie

run() {
  group("[optional parameters]", (){
    setUp(() => _prints = []);

    test('required only', (){
      // START:no_options
      good_day("Bob");
      // Good day, Bob.
      // END:no_options
      expect(
        _prints,
        equals(['Good day, Bob.'])
      );
    });

    test('optional named parameter', (){
      // START:no_options
      good_day("Bob", emphatic: true);
      // Good day, Bob.
      // I said good day!
      // END:no_options

      expect(_prints, equals([
        'Good day, Bob.',
        'I said good day!'
      ]));
    });

    test('default optional parameter', (){
      // START:profile_bob
      profile("Bob");
      // Name: Bob
      //   personal hero: Weird Al
      // END:profile_bob

      expect(_prints, equals([
        'Name: Bob',
        '  personal hero: Weird Al'
      ]));
    });

    test('specifying a default, optional parameter', (){
      // START:specify_default_named_param
      profile("Bob", favorite_color: 'Purple', hero: 'Frank Drebin');
      // Name: Bob
      //   personal hero: Frank Drebin
      //   favorite color: Purple
      // END:specify_default_named_param

      expect(_prints, equals([
        'Name: Bob',
        '  personal hero: Frank Drebin',
        '  favorite color: Purple'
      ]));
    });


    test('optional positional parameters', (){
      // START:optional_positional_params
      movie("The Naked Gun");
      // Great movie: The Naked Gun
      //   Starring: Leslie Nielson
      // END:optional_positional_params

      expect(_prints, equals([
        'Great movie: The Naked Gun',
        '  Starring: Leslie Nielson'
      ]));
    });

    test('specifying optional, positional parameters', (){
      // START:optional_positional_params
      movie("Airplane!", "Robert Hays", "Leslie Nielson");
      // Great movie: The Naked Gun
      //   Starring: Robert Hays
      //   Co-starring: Leslie Nielson
      // END:optional_positional_params

      expect(_prints, equals([
        'Great movie: Airplane!',
        '  Starring: Robert Hays',
        '  Co-starring: Leslie Nielson'
      ]));
    });
  });
}
