library main;

// START:imports_and_main_initial
import 'dart:html';
import 'dart:async';
import 'dart:collection';
import 'dart:convert';
// START:main
// START:main_initial
main() {
  var my_comics_collection = new ComicsCollection();
  new ComicsView(
    el: document.query('#comics-list'),
    collection: my_comics_collection
  );
  my_comics_collection.fetch();
  // END:main_initial
  // END:imports_and_main_initial

  new CreateComicView(
    el: document.query('#add-comic'),
    collection: my_comics_collection
  );
// START:main_close
}
// END:main_close
// END:main

// START:comics_collection
// START:comics_collection_intro
// START:comics_collection_declaration_elide
// START:comics_collection_declaration
class ComicsCollection extends IterableBase {
// END:comics_collection_declaration
  // ...
// END:comics_collection_declaration_elide
  StreamController _onLoad, _onAdd;
  List models;

  // Constructor method
  ComicsCollection() {
    _onLoad = new StreamController.broadcast();
    _onAdd = new StreamController.broadcast();
    models = [];
  }
  // END:comics_collection_intro

  Stream get onLoad => _onLoad.stream;
  Stream get onAdd => _onAdd.stream;

  // START:collection_methods
  String get url => 'http://localhost:31337/widgets';
  Iterator get iterator => models.iterator;
  // END:collection_methods


  // START:fetch
  void fetch() {
    var req = new HttpRequest();
    req.onLoad.listen((event) {
      var list = JSON.decode(req.responseText);
      _handleOnLoad(list);
    });
    req.open('get', url);
    req.send();
  }
  // END:fetch

  // START:_handleOnLoad
  _handleOnLoad(list) {
    list.forEach((attrs) {
      var new_model = new ComicBook(attrs, collection: this);
      models.add(new_model);
    });
    _onLoad.add(this);
  }
  // END:_handleOnLoad

  // START:create
  create(attrs) {
    var new_model = new ComicBook(attrs, collection: this);
    new_model.save((event) {
      this.add(new_model);
    });
  }
  add(model) {
    models.add(model);
    _onAdd.add(model);
  }
// START:comics_collection_elide_close
  // ...
// START:comics_collection_close
}
// END:comics_collection_close
// END:comics_collection_elide_close
// END:create
// START:comics_collection

// START:comic_model_operator
// START:comic_model_intro
class ComicBook {
  Map attributes;
  ComicsCollection collection;

  StreamController _onSave, _onDelete;
  Stream onSave, onDelete;

  ComicBook(this.attributes, {this.collection}) {
    _onSave = new StreamController.broadcast();
    onSave = _onSave.stream;
    _onDelete = new StreamController.broadcast();
    onDelete = _onDelete.stream;
  }
  // END:comic_model_intro

  operator [](attr) => attributes[attr];
  // END:comic_model_operator

  // START:model_url
  get id => attributes['id'];
  String get url => isSaved() ?
    "$urlRoot/$id" : urlRoot;

  String get urlRoot => (collection == null) ?
    "" : collection.url;

  bool isSaved() => id != null;
  // END:model_url

  // START:model_save
  save([callback]) {
    var req = new HttpRequest(),
        json = JSON.encode(attributes);
    req.onLoad.listen((load_event) {
      var request = load_event.target;
      attributes = JSON.decode(request.responseText);
      _onSave.add(this);
      if (callback != null) callback(this);
    });
    req.open('post', url);
    req.setRequestHeader('Content-type', 'application/json');
    req.send(json);
  }
  // END:model_save

  // START:model_delete
  delete([callback]) {
    var req = new HttpRequest();
    req.onLoad.listen((load_event) {
      var request = load_event.target;
      _onDelete.add(this);
      if (callback != null) callback(this);
    });
    req.open('delete', url);
    req.send();
  }
  // END:model_delete
// START:comic_model_close
}
// END:comic_model_close

// START:create_comic_view_constructor
class CreateComicView {
  var el, model, collection;
  Element form;
  
  CreateComicView({this.el, this.model, this.collection}) {
    el.onClick.listen((event) => toggleForm());
    attachForm();
  }
  // END:create_comic_view_constructor

  // START:toggle_form
  toggleForm() {
    if (form.style.display == 'none') {
      form.style.display = 'block';
    }
    else {
      form.style.display = 'none';
    }
  }
  // END:toggle_form

  // START: attach_form
  attachForm() {
    form = new Element.html("<div>${template}</div>");
    form.style.display = 'none';
    el.parent.children.add(form);
    InputElement titleInput = form.query('input[name=title]'),
                 authorInput = form.query('input[name=author]');
    // handle create form submission
    form.query('form').onSubmit.listen((event) {
      collection.create({
        'title': titleInput.value,
        'author': authorInput.value
      });
      form.
        queryAll('input[type=text]').
        forEach((InputElement input) => input.value = '');
      toggleForm();
      event.preventDefault();
    });

    // handle clicks on the Cancel link
    form.query('a').onClick.listen((event) {
      toggleForm();
      event.preventDefault();
    });
  }
  // END: attach_form

  get template {
    return """
      <form action="comics" id="new-comic-form">
        <p>
          <label>Title<br>
            <input type="text" name="title">
          </label>
        </p>

        <p>
          <label>Author<br>
            <input type="text" name="author">
          </label>
        </p>

        <p>
          <input type="submit" value="Create!"><br>
          <a href="#">Cancel</a>
        </p>
      </form>
    """;
  }
// START:create_comic_view_close
}
// END:create_comic_view_close

// START:comic_view_intro
// START:comic_view_declaration
class ComicsView {
  // END:comic_view_declaration
  var el, model, collection;
  ComicsView({this.el, this.model, this.collection}) {
    if (collection == null) return;
    collection.onLoad.listen((event) => render());
    collection.onAdd.listen((event) => render());
  }
  // END:comic_view_intro

  // START:render
  // ...
  render() {
    el.innerHtml = template(collection);
  }
  template(list) {
    if (list.length == 0) return '';

    var html = '';
    list.forEach((comic) {
      html += _singleComicBookTemplate(comic);
    });
    return html;
  }

  _singleComicBookTemplate(comic) {
    return """
      <li id="${comic['id']}">
        ${comic['title']}
        (${comic['author']})
        <a href="#" class="delete">[delete]</a>
      </li>""";
  }
// START:comic_view_close
}
// END:comic_view_close
// END:render
