import 'dart:io';
import 'dart:convert';

import 'package:dirty/dirty.dart';
import 'package:uuid/uuid.dart';

main() {
  HttpServer app = new HttpServer();

  new StaticFiles(app);

  app.addRequestHandler(
    (req) => req.method == 'GET' && req.path == '/comics',
    Comics.index
  );

  app.addRequestHandler(
    (req) => req.method == 'POST' && req.path == '/comics',
    Comics.post
  );

  app.addRequestHandler(
    (req) => req.method == 'DELETE' &&
             new RegExp(r"^/comics/[-\w\d]+$").hasMatch(req.path),
    Comics.delete
  );

  var port = 8000;
  if (Platform.environment['ENV'] == 'test') {
    port = 9999;
    new StaticFiles(app, dir:'test', mountPoint: '/test');
    Comics.filename = 'test.db';
    Comics.db.clear();
  }
  if (Platform.environment['PORT'] != null)  {
    port = int.parse(Platform.environment['PORT']);
  }
  app.listen('0.0.0.0', port);
  print('Server started on port: ${port}');
}

class Comics {
  static Uuid uuid = new Uuid();
  static String filename = 'dart_comics.db';
  static Dirty db = new Dirty(filename);

  static index(req, res) {
    res.headers.contentType = 'application/json';
    res.outputStream.writeString(JSON.encode(db.values));
    res.outputStream.close();
  }

  static post(req, res) {
    var input = new StringInputStream(req.inputStream);
    var post_data = '';

    input.onLine = () {
      var line = input.readLine();
      post_data = post_data.concat(line);
    };

    input.onClosed = () {
      var graphic_novel = JSON.decode(post_data);
      graphic_novel['id'] = uuid.v1();

      db[graphic_novel['id']] = graphic_novel;

      res.statusCode = 201;
      res.headers.contentType = 'application/json';

      res.outputStream.writeString(JSON.encode(graphic_novel));
      res.outputStream.close();
    };
  }

  static delete(req, res) {
    var r = new RegExp(r"^/comics/([-\w\d]+)");
    var id = r.firstMatch(req.path)[1];

    db.remove(id);

    res.outputStream.writeString('{}');
    res.outputStream.close();
  }
}

class StaticFiles {
  var dir, mountPoint;

  StaticFiles(app, {this.dir:"public", this.mountPoint:""}) {
    app.addRequestHandler(this.matcher, this.handler);
  }

  matcher(req) {
    if (req.method != 'GET') return false;

    String path = publicPath(req.path);
    if (path == null) return false;

    req.session().data = {'path': path};
    return true;
  }

  handler(req, res) {
    var file = new File(req.session().data['path']);
    var stream = file.openInputStream();
    stream.pipe(res.outputStream);
  }

  String publicPath(String path) {
    var local_path = path.replaceFirst(mountPoint, "");
    if (pathExists("$dir/$local_path")) return "$dir/$local_path";
    if (pathExists("$dir/$local_path/index.html")) return "$dir/$local_path/index.html";
  }

  bool pathExists(String path) => new File(path).existsSync();
}
